<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'VP_Woo_Pont_Automations', false ) ) :

	class VP_Woo_Pont_Automations {

		//Setup triggers
		public static function init() {

			//When order created
			add_action( 'woocommerce_checkout_order_processed', array( __CLASS__, 'on_order_created' ), 10, 3 );

			//On successful payment
			add_action( 'woocommerce_payment_complete', array( __CLASS__, 'on_payment_complete' ) );

			//On status change
			add_action( 'woocommerce_order_status_changed', array( __CLASS__, 'on_status_change' ), 10, 3 );

		}

		public static function on_order_created($order_id, $posted_data, $order) {
			$automations = self::find_automations($order_id, 'order_created');
		}

		public static function on_payment_complete( $order_id ) {
			$automations = self::find_automations($order_id, 'payment_complete');

		}

		public static function on_status_change( $order_id, $old_status, $new_status ) {
			$automations = self::find_automations($order_id, $new_status);
		}

		public static function find_automations($order_id, $trigger) {

			//Get main data
			$order = wc_get_order($order_id);
			$automations = get_option('vp_woo_pont_automations');
			$order_details = VP_Woo_Pont_Conditions::get_order_details($order, 'automations');

			//We will return the matched automations at the end
			$final_automations = array();

			//Loop through each automation
			foreach ($automations as $automation_id => $automation) {

				//Check if trigger is a match. If not, just skip
				if(str_replace( 'wc-', '', $automation['trigger'] ) != str_replace( 'wc-', '', $trigger )) {
					continue;
				}

				//If this is based on a condition
				if($automation['conditional']) {

					//Compare conditions with order details and see if we have a match
					$automation_is_a_match = VP_Woo_Pont_Conditions::match_conditions($automations, $automation_id, $order_details);

					//If its not a match, continue to next not
					if(!$automation_is_a_match) continue;

					//If its a match, add to found automations
					$final_automations[] = $automation;

				} else {
					$final_automations[] = $automation;
				}

			}

			//If we found some automations, try to generate documents
			if(count($final_automations) > 0) {

				//Loop through documents(usually it will be only one, but who knows)
				if($order->get_meta('_vp_woo_pont_provider')) {
					self::run_automations($order_id, $final_automations);
				}

			}

			return $final_automations;
		}

		public static function run_automations($order_id, $automations) {

			//Get data
			$order = wc_get_order($order_id);

			//If generate in the background
			$deferred = true;

			//Don't create deferred if we are in an admin page and only mark one order completed
			if(is_admin() && isset( $_GET['action']) && $_GET['action'] == 'woocommerce_mark_order_status') {
				$deferred = false;
			}

			//Don't defer if we are just changing one or two order status using bulk actions
			if(is_admin() && isset($_GET['_wp_http_referer']) && isset($_GET['post']) && count($_GET['post']) < 3) {
				$deferred = false;
			}

			//Loop through automations
			foreach ($automations as $automation) {

				//Check if it was already generated
				$generated = ($order->get_meta('_vp_woo_pont_parcel_pdf'));

				//Skip, if already generated
				if($generated) continue;

				//For postapont orders, just use the postapont part from the provider name, its the same for all
				$provider = $order->get_meta('_vp_woo_pont_provider');
				if(strpos($provider, 'postapont') !== false) {
					$provider = 'postapont';
				}

				//If we are still here, we can generate the actual document
				if($deferred) {
					WC()->queue()->add( 'vp_woo_pont_generate_label_async', array('order_id' => $order_id, 'provider' => $provider), 'vp-woo-pont' );
				} else {
					$return_info = VP_Woo_Pont()->labels->generate_label($order_id, $provider);
				}
			}
		}
	}

	VP_Woo_Pont_Automations::init();

endif;
