<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

//Load PDF API
use iio\libmergepdf\Merger;
use iio\libmergepdf\Driver\TcpdiDriver;

if ( ! class_exists( 'VP_Woo_Pont_Labels', false ) ) :

	class VP_Woo_Pont_Labels {
		public $bulk_actions = array();
		public $supported_providers = array('foxpost', 'packeta', 'gls');

		public function __construct() {

			//Set bulk action values
			$this->bulk_actions = array(
				'vp_woo_pont_generate_labels' => __( 'Generate shipping labels', 'vp-woo-pont' ),
				'vp_woo_pont_print_labels' => __( 'Print shipping labels', 'vp-woo-pont' ),
				'vp_woo_pont_download_labels' => __( 'Download shipping labels', 'vp-woo-pont' ),
			);

			//Add bulk action to print labels
			add_filter( 'bulk_actions-edit-shop_order', array( $this, 'add_bulk_options'), 20, 1);
			add_filter( 'handle_bulk_actions-edit-shop_order', array( $this, 'handle_bulk_actions'), 10, 3 );
			add_action( 'admin_notices', array( $this, 'bulk_actions_results') );

			//Create metabox for labels on order edit page
			add_action( 'add_meta_boxes', array( $this, 'add_metabox' ), 10, 2 );

			//Add label and package number to orders table
			add_action( 'manage_shop_order_posts_custom_column', array( $this, 'add_listing_actions' ), 20 );

			//Ajax function to generate and remove labels
			add_action( 'wp_ajax_vp_woo_pont_generate_label', array( $this, 'ajax_generate_label' ) );
			add_action( 'wp_ajax_vp_woo_pont_void_label', array( $this, 'ajax_void_label' ) );

			//Update PDF link via heartbeat
			add_filter( 'heartbeat_received', array( $this, 'add_pdf_to_heartbeat'), 10, 2 );

			//Include invoices in emails
			if(VP_Woo_Pont_Helpers::get_option('email_tracking_number')) {
				if(VP_Woo_Pont_Helpers::get_option('email_tracking_number_pos', 'beginning') == 'beginning') {
					add_action('woocommerce_email_before_order_table', array( $this, 'email_attachment'), 10, 4);
					add_action('woocommerce_subscriptions_email_order_details', array( $this, 'email_attachment'), 10, 4);
				} else {
					add_action('woocommerce_email_customer_details', array( $this, 'email_attachment'), 30, 4);
				}
			}

		}

		public function add_bulk_options( $actions ) {
			$actions['vp_woo_pont_generate_labels'] = __( 'Generate shipping labels', 'vp-woo-pont' );
			$actions['vp_woo_pont_print_labels'] = __( 'Print shipping labels', 'vp-woo-pont' );
			$actions['vp_woo_pont_download_labels'] = __( 'Download shipping labels', 'vp-woo-pont' );
			return $actions;
		}

		public function handle_bulk_actions( $redirect_to, $action, $post_ids ) {

			//Check if we are processing a vp_woo_pont bulk request
			if( strpos($action, 'vp_woo_pont') !== false ) {
				$query_args = array_keys($this->bulk_actions);
				$action_params = explode('_', $action);
				$action = $action_params[3];

				//If we are downloading or printing
				if ( in_array($action, array('print', 'download'))) {

					//Remove existing params from url
					$redirect_to = remove_query_arg(array('vp_woo_pont_bulk_action', 'vp_woo_pont_results_bulk_count', 'vp_woo_pont_results_bulk_pdf'), $redirect_to);

					//Create bulk pdf file
					$bulk_pdf_file = $this->get_pdf_file_path('bulk', 0);

					if($action == 'download' && VP_Woo_Pont_Helpers::get_option('bulk_download_zip', 'no') == 'yes') {

						//Create an object from the ZipArchive class.
						$zipArchive = new ZipArchive();

						//The full path to where we want to save the zip file.
						$bulk_pdf_file['path'] = str_replace('.pdf', '.zip', $bulk_pdf_file['path']);
						$bulk_pdf_file['name'] = str_replace('.pdf', '.zip', $bulk_pdf_file['name']);

						//Call the open function.
						$status = $zipArchive->open($bulk_pdf_file['path'], ZipArchive::CREATE | ZipArchive::OVERWRITE);

						//An array of files that we want to add to our zip archive.
						foreach ( $post_ids as $order_id ) {
							$order = wc_get_order($order_id);
							$pdf_file = $this->generate_download_link($order, true);
							if($pdf_file && file_exists($pdf_file)) {
								$new_filename = substr($pdf_file, strrpos($pdf_file,'/') + 1);
								$zipArchive->addFile($pdf_file, $new_filename);
								$processed[] = $order_id;
							}
						}

						//Bail if theres no $documents
						if(!$processed) {
							return $redirect_to;
						}

						//Finally, close the active archive.
						$zipArchive->close();

					} else {

						//Init PDF merger
						require_once plugin_dir_path(__FILE__) . '../vendor/autoload.php';
						$merger = new Merger(new TcpdiDriver);
						$processed = array();

						//Process selected posts
						foreach ( $post_ids as $order_id ) {
							$order = wc_get_order($order_id);
							$pdf_file = $this->generate_download_link($order, true);
							if($pdf_file && file_exists($pdf_file)) {
								$merger->addFile($pdf_file);
								$processed[] = $order_id;
							}
						}

						//Bail if theres no $documents
						if(!$processed) {
							return $redirect_to;
						}

						//Create bulk pdf file
						$merged_pdf_file = $merger->merge();

						//Store PDF
						global $wp_filesystem;
						if ( !$wp_filesystem ) WP_Filesystem();
						$wp_filesystem->put_contents( $bulk_pdf_file['path'], $merged_pdf_file );

					}

					//Set redirect url that will show the download message notice
					$redirect_to = add_query_arg( array('vp_woo_pont_bulk_action' => $action, 'vp_woo_pont_results_bulk_count' => count( $processed ), 'vp_woo_pont_results_bulk_pdf' => urlencode($bulk_pdf_file['name'])), $redirect_to );
					return $redirect_to;

				} else if ($action == 'generate') {

					//Remove existing params from url
					$redirect_to = remove_query_arg($query_args, $redirect_to);
					$redirect_to = remove_query_arg(array('vp_woo_pont_bulk_action', 'vp_woo_pont_results_bulk_count', 'vp_woo_pont_results_bulk_pdf'), $redirect_to);

					//Grouped orders
					$needs_processing = array();
					$processed = array();

					//Check if we need to defer
					$defer_limit = apply_filters('vp_woo_pont_generate_defer_limit', 2);

					//Loop throguh order ids, check if its vp pont and group by providers and prepare data for generating labels
					foreach ($post_ids as $order_id) {

						//Check if its a vp_pont order
						$order = wc_get_order($order_id);
						$provider = $order->get_meta('_vp_woo_pont_provider');

						//If it is, check if a label is generated already or not
						if($provider && !$this->is_label_generated($order) && in_array($provider, $this->supported_providers)) {

							//For postapont orders, just use the postapont part from the provider name, its the same for all
							if(strpos($provider, 'postapont') !== false) {
								$provider = 'postapont';
							}

							//Append to future processing
							$needs_processing[] = array('order_id' => $order_id, 'provider' => $provider);

						}

					}

					//If we need to process more than the defer limit, do it in the backgorund
					if(count($needs_processing) > $defer_limit) {
						foreach ( $needs_processing as $label_data ) {
							WC()->queue()->add( 'vp_woo_pont_generate_label_async', $label_data, 'vp-woo-pont' );
							$processed[] = $label_data['order_id'];
						}
					} else {
						foreach ( $needs_processing as $label_data ) {
							$this->generate_label($label_data['order_id'], $label_data['provider']);
							$processed[] = $label_data['order_id'];
						}
					}

					//Set redirect url that will show the download message notice
					$redirect_to = add_query_arg( array('vp_woo_pont_bulk_action' => $action, 'vp_woo_pont_bulk_action_results' => implode('|', $processed)), $redirect_to );
					return $redirect_to;

				} else {
					return $redirect_to;
				}

			} else {
				return $redirect_to;
			}

		}

		public function bulk_actions_results() {
			if(isset($_REQUEST['vp_woo_pont_bulk_action'])) {

				//If its a print or download request
				$action = sanitize_text_field($_REQUEST['vp_woo_pont_bulk_action']);

				if ( in_array($action, array('print', 'download')) ) {
					$print_count = intval( $_REQUEST['vp_woo_pont_results_bulk_count'] );

					$paths = $this->get_pdf_file_path('bulk', 0);
					$pdf_file_name = esc_attr( $_REQUEST['vp_woo_pont_results_bulk_pdf'] );
					$pdf_file_url = $paths['baseurl'].$pdf_file_name;

					include( dirname( __FILE__ ) . '/views/html-notice-bulk.php' );
				}

				if ( $action == 'generate') {
					$documents = explode('|', $_REQUEST['vp_woo_pont_bulk_action_results']);
					include( dirname( __FILE__ ) . '/views/html-notice-bulk.php' );
				}

			}
		}

		public function generate_label($order_id, $provider = false) {

			//Create response object
			$response = array();
			$response['error'] = false;
			$response['messages'] = array();

			//Get order data
			$order = wc_get_order($order_id);

			//If provider not set, find it
			if(!$provider) $provider = $order->get_meta('_vp_woo_pont_provider');

			//Set label data
			$data = $this->prepare_order_data($order);
			$data = apply_filters('vp_woo_pont_prepare_order_data_for_label', $data, $order, $provider);

			//Try to generate the label with the provider class
			$label = VP_Woo_Pont()->providers[$provider]->create_label($data);

			//Check for errors
			if(is_wp_error($label)) {
				$response['error'] = true;
				$response['messages'][] = $label->get_error_message();

				//Log error message
				$order->add_order_note(sprintf(esc_html__('Unable to create shipping label. Error code: %s', 'vp-woo-pont'), urldecode($label->get_error_code())));

				return $response;
			}

			//If generation is still pending(foxpost for example only works async and takes a few minutes)
			if(isset($label['pending']) && $label['pending']) {

				//Update metadata
				$order->update_meta_data('_vp_woo_pont_parcel_pending', true);
				$order->save();

				//Save order note
				$order->add_order_note(esc_html__('Shipping label generating is in progress. It might take a couple of minutes to show up here.', 'vp-woo-pont'));

				//Create response
				$response['pending'] = true;
				$response['messages'][] = esc_html__('Shipping label generating is in progress.','vp-woo-pont');

			} else {

				//If all went good, store the PDF file and shipment info
				$order->update_meta_data('_vp_woo_pont_parcel_id', $label['id']);
				$order->update_meta_data('_vp_woo_pont_parcel_pdf', $label['pdf']);
				$order->update_meta_data('_vp_woo_pont_parcel_number', $label['number']);
				$order->save();

				//Save order note
				$order->add_order_note(sprintf(esc_html__('Shipping label generated successfully. ID number: %s', 'vp-woo-pont'), $label['number']));

				//Create response
				$response['number'] = $label['number'];
				$response['pdf'] = $this->generate_download_link($order);
				$response['messages'][] = esc_html__('Shipping label generated successfully.','vp-woo-pont');

				//So developers can hook in here
				do_action('vp_woo_pont_label_created', $order, $label, $provider);

			}

			//Change order status if needed based on settings
			//TODO

			//Return response
			return $response;

		}

		public function void_label($order_id, $provider = false) {

			//Create response object
			$response = array();
			$response['error'] = false;
			$response['messages'] = array();

			//Get order data
			$order = wc_get_order($order_id);

			//If provider not set, find it
			if(!$provider) $provider = $order->get_meta('_vp_woo_pont_provider');

			//Set label data
			$data = $this->prepare_order_data($order);
			$data['parcel_number'] = $order->get_meta('_vp_woo_pont_parcel_number');
			$data['parcel_id'] = $order->get_meta('_vp_woo_pont_parcel_id');
			$data = apply_filters('vp_woo_pont_prepare_order_data_for_void_label', $data, $order, $provider);

			//Try to generate the label with the provider class
			$label = VP_Woo_Pont()->providers[$provider]->void_label($data);

			//Check for errors
			if(is_wp_error($label)) {
				$response['error'] = true;
				$response['messages'][] = $label->get_error_message();

				//Log error message
				$order->add_order_note(sprintf(esc_html__('Unable to void the shipping label. Error code: %s', 'vp-woo-pont'), urldecode($label->get_error_code())));

				return $response;
			}

			//If all went good, store the PDF file and shipment info
			$order->delete_meta_data('_vp_woo_pont_parcel_id');
			$order->delete_meta_data('_vp_woo_pont_parcel_pdf');
			$order->delete_meta_data('_vp_woo_pont_parcel_number');
			$order->delete_meta_data('_vp_woo_pont_parcel_pending');
			$order->save();

			if(isset($label['deleted_locally'])) {

				//Save order note
				$order->add_order_note(sprintf(esc_html__("Shipping label removed from the order only. You still need to remove it in the provider's system. ID number: %s", 'vp-woo-pont'), $data['label_id']));

				//Create response
				$response['messages'][] = esc_html__("Shipping label removed from WooCommerce. You still need to remove it in the provider's system.",'vp-woo-pont');

			} else {

				//Save order note
				$order->add_order_note(sprintf(esc_html__('Shipping label removed successfully. ID number: %s', 'vp-woo-pont'), $data['label_id']));

				//Create response
				$response['messages'][] = esc_html__('Shipping label removed successfully.','vp-woo-pont');

			}

			//So developers can hook in here
			do_action('vp_woo_pont_label_removed', $order, $data, $provider);

			//Return response
			return $response;

		}

		//Function to create an array that can be used by all providers to generate labels
		public function prepare_order_data($order) {

			//Calculate weight
			$total_weight = 0;
			foreach ( $order_items as $item_id => $product_item ) {
				$product = $product_item->get_product();
				if($product) {
					$product_weight = $product->get_weight();
					$quantity = $product_item->get_quantity();
					$total_weight += floatval( $product_weight * $quantity );
				}
			}

			//Compile data required for labels
			$data = array(
				'order_id' => $order->get_id(),
				'order_number' => $order->get_order_number(),
				'order' => $order,
				'customer' => array(
					'name' => $order->get_formatted_billing_full_name(),
					'phone' => $this->format_phone_number($order->get_billing_phone()),
					'email' => $order->get_billing_email(),
					'first_name' => $order->get_billing_first_name(),
					'last_name' => $order->get_billing_last_name(),
					'company' => $order->get_billing_company()
				),
				'package' => array(
					'total' => $order->get_total(),
					'total_rounded' => 5 * round($order->get_total() / 5),
					'weight' => $total_weight,
					'cod' => ($order->get_payment_method() == 'cod'),
					'currency' => $order->get_currency()
				),
				'point_id' => $order->get_meta('_vp_woo_pont_point_id')
			);

			return $data;
		}

		//Create download link for the PDF file
		public function generate_download_link( $order, $absolute = false) {
			if($order) {
				$pdf_name = '';
				$pdf_name = $order->get_meta('_vp_woo_pont_parcel_pdf');

				if($pdf_name) {
					$paths = $this->get_pdf_file_path();
					if($absolute) {
						$pdf_file_url = $paths['basedir'].$pdf_name;
					} else {
						$pdf_file_url = $paths['baseurl'].$pdf_name;
					}
					return $pdf_file_url;
				} else {
					return false;
				}
			} else {
				return false;
			}
		}

		//Get file path for pdf files
		public static function get_pdf_file_path($provider = 'postapont', $order_id = 0) {
			$upload_dir = wp_upload_dir( null, false );
			$basedir = $upload_dir['basedir'] . '/vp-woo-pont-labels/';
			$baseurl = $upload_dir['baseurl'] . '/vp-woo-pont-labels/';
			$random_file_name = substr(md5(rand()),5);
			$pdf_file_name = implode( '-', array( $provider, $order_id, $random_file_name ) ).'.pdf';
			$file_dir = $basedir;

			//Group by year and month if needed
			if (get_option('uploads_use_yearmonth_folders') ) {
				$time = current_time( 'mysql' );
				$y = substr( $time, 0, 4 );
				$m = substr( $time, 5, 2 );
				$subdir = "/$y/$m";
				$pdf_file_name = $y.'/'.$m.'/'.$pdf_file_name;
				$file_dir = $basedir.$y.'/'.$m.'/';
			}

			return array('name' => $pdf_file_name, 'filedir' => $file_dir, 'path' => $basedir.$pdf_file_name, 'baseurl' => $baseurl, 'basedir' => $basedir);
		}

		//Meta box on order page
		public function add_metabox( $post_type, $post ) {
			$order = wc_get_order($post->ID);
			if($order && $order->get_meta('_vp_woo_pont_provider')) {
				add_meta_box('vp_woo_pont_metabox', __('Pickup point', 'vp-woo-pont'), array( $this, 'render_metabox_content' ), 'shop_order', 'side');
			}
		}

		//Render metabox content
		public function render_metabox_content($post) {
			$order = wc_get_order($post->ID);
			include( dirname( __FILE__ ) . '/views/html-metabox.php' );
		}

		//Ajax function to create label
		public function ajax_generate_label() {
			check_ajax_referer( 'vp_woo_pont_manage', 'nonce' );
			$order_id = intval($_POST['order']);
			$response = $this->generate_label($order_id);
			wp_send_json_success($response);
		}

		//Ajax function to create label
		public function ajax_void_label() {
			check_ajax_referer( 'vp_woo_pont_manage', 'nonce' );
			$order_id = intval($_POST['order']);
			$response = $this->void_label($order_id);
			wp_send_json_success($response);
		}

		//Return the pdf label info with heartbeat
		public function add_pdf_to_heartbeat($response, $data) {

			//If we didn't receive our data, don't send any back.
			if ( empty( $data['vp_woo_pont_label_generate'] )) {
				return $response;
			}

			//Only on the order page though
			$order_id = sanitize_text_field($data['vp_woo_pont_order_id']);
			$order = wc_get_order($order_id);

			//Check if its still pending or not, if not, return the PDF
			if($order && !$order->get_meta('_vp_woo_pont_parcel_pending')) {
				$pdf_link = $this->generate_download_link($order);
				$response['vp_woo_pont_label_pdf'] = $pdf_link;
			}

			return $response;
		}

		//Save PDF file
		public static function save_pdf_file($pdf, $pdf_file) {

			//If upload folder doesn't exists, create it with an empty index.html file
			$file = array(
				'base' 		=> $pdf_file['filedir'],
				'file' 		=> 'index.html',
				'content' 	=> ''
			);

			if ( wp_mkdir_p( $file['base'] ) && ! file_exists( trailingslashit( $file['base'] ) . $file['file'] ) ) {
				if ( $file_handle = @fopen( trailingslashit( $file['base'] ) . $file['file'], 'w' ) ) {
					fwrite( $file_handle, $file['content'] );
					fclose( $file_handle );
				}
			}

			//Make a GET request and store the pdf file
			file_put_contents($pdf_file['path'], $pdf);

		}

		//Helper function to force correct phone number format for FoxPost API
		public function format_phone_number($number) {
			$number = preg_replace( '/[^0-9]/', '', $number );
			$number = str_replace('+36', '', $number);
			$prefixes = array('+36', '36', '06');
			foreach ($prefixes as $prefix) {
				if (substr($number, 0, strlen($prefix)) == $prefix && strlen($number) > 7) {
					$number = substr($number, strlen($prefix));
				}
			}
			$number = '+36'.$number;
			return $number;
		}

		//Check if it was already generated or not
		public function is_label_generated( $order) {
			return ($order->get_meta('_vp_woo_pont_parcel_pdf'));
		}

		//Function to get a tracking link URL
		public function get_tracking_link($order) {
			$link = '';

			if($this->is_label_generated($order)) {
				$provider_id = $order->get_meta('_vp_woo_pont_provider');
				$parcel_number = $order->get_meta('_vp_woo_pont_parcel_number');
				$link = VP_Woo_Pont()->providers[$provider_id]->get_tracking_link($parcel_number);
			}

			return esc_url($link);
		}

		//Add icon to order list to show invoice
		public function add_listing_actions( $column ) {
			global $the_order;

			if ( 'shipping_address' === $column) {
				if($provider_id = $the_order->get_meta('_vp_woo_pont_provider')) {
					$providers = VP_Woo_Pont_Helpers::get_supported_providers();
					$provider_name = $providers[$provider_id];
					?>
					<div class="vp-woo-pont-order-column">
						<?php if($this->is_label_generated($the_order)): ?>
							<a target="_blank" href="<?php echo $this->get_tracking_link($the_order); ?>" class="vp-woo-pont-order-column-tracking help_tip" data-tip="<?php esc_attr_e('Tracking number', 'vp-woo-pont'); ?>">
								<i class="vp-woo-pont-provider-icon-<?php echo esc_attr($provider_id); ?>"></i>
								<span><?php echo esc_html($the_order->get_meta('_vp_woo_pont_parcel_number')); ?></span>
							</a>
							<a target="_blank" href="<?php echo $this->generate_download_link($the_order); ?>" class="vp-woo-pont-order-column-pdf help_tip" data-tip="<?php esc_attr_e('Shipping label', 'vp-woo-pont'); ?>">
								<i></i>
								<span><?php echo esc_html($the_order->get_meta('_vp_woo_pont_parcel_id')); ?></span>
							</a>
						<?php else: ?>
							<span class="vp-woo-pont-order-column-label">
								<i class="vp-woo-pont-provider-icon-<?php echo esc_attr($provider_id); ?>"></i>
								<span><?php echo esc_html($provider_name); ?></span>
							</span>
						<?php endif; ?>
					</div>
				<?php
				}
			}
		}

		//Email attachment
		public function email_attachment($order, $sent_to_admin, $plain_text, $email){
			$order_id = $order->get_id();
			$order = wc_get_order($order_id);

			if(isset($email->id) && is_a( $order, 'WC_Order' ) && $this->is_label_generated($order)) {
				$tracking_url = $this->get_tracking_link($order);
				$tracking_number = $order->get_meta('_vp_woo_pont_parcel_number');
				$tracking_link = '<a target="_blank" href="'.esc_url($tracking_url).'">'.esc_html($tracking_number).'</a>';
				$text = VP_Woo_Pont_Helpers::get_option('email_tracking_number_desc', __('You can track the order by clicking on the tracking number: {tracking_number}', 'vp-woo-pont'));
				$params = array( 'order' => $order, 'tracking_url' => $tracking_url, 'tracking_link' => $tracking_link, 'tracking_number' => $tracking_number, 'tracking_text' => $text );

				if($plain_text) {
					wc_get_template( 'emails/plain/email-vp-woo-pont-section.php', $params, '', plugin_dir_path( __FILE__ ) );
				} else {
					wc_get_template( 'emails/email-vp-woo-pont-section.php', $params, '', plugin_dir_path( __FILE__ ) );
				}
			}
		}

	}

endif;
