<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'VP_Woo_Pont_Pro', false ) ) :

	class VP_Woo_Pont_Pro {

		public static function init() {

			//Check and save PRO version
			add_action( 'wp_ajax_vp_woo_pont_pro_check', array( __CLASS__, 'pro_check' ) );
			add_action( 'wp_ajax_vp_woo_pont_pro_deactivate', array( __CLASS__, 'pro_deactivate' ) );

			//Hide rate request
			add_action( 'wp_ajax_vp_woo_pont_hide_rate_request', array( __CLASS__, 'hide_rate_request' ) );

		}

		public static function get_license_key() {
			return VP_Woo_Pont::$license->get_input_license_value('deactive', VP_Woo_Pont::$license);
		}

		public static function pro_check() {
			if ( !current_user_can( 'edit_shop_orders' ) ) {
				wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
			}

			//Try to activate with appsero
			VP_Woo_Pont::$license->license_form_submit([
			    '_nonce'      => wp_create_nonce( VP_Woo_Pont::$appsero->name ),
			    '_action'     => 'active',
			    'license_key' => sanitize_text_field($_POST['key']),
			]);

			if ( ! VP_Woo_Pont::$license->error ) {
			    wp_send_json_success();
			} else {
				wp_send_json_error(array(
					'message' => __('Unable to activate the PRO version. Please make sure that the entered data is correct.', 'vp-woo-pont')
				));
			}

		}

		public static function pro_deactivate() {
			if ( !current_user_can( 'edit_shop_orders' ) ) {
				wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
			}

			//Try to activate with appsero
			VP_Woo_Pont::$license->license_form_submit([
					'_nonce'      => wp_create_nonce( VP_Woo_Pont::$appsero->name ),
					'_action'     => 'deactive',
			]);

			if ( ! VP_Woo_Pont::$license->error ) {
				wp_send_json_success();
			} else {
				wp_send_json_error(array(
					'message' => __('Unable to deactivate the license. Please contact support for help.', 'vp-woo-pont')
				));
			}

		}

		//Save an option with ajax, so the rate request widget can be hidden
		public static function hide_rate_request() {
			check_ajax_referer( 'vp-woo-pont-hide-rate-request', 'nonce' );
			update_option('_vp_woo_pont_hide_rate_request', true);
			wp_send_json_success();
		}

	}

	VP_Woo_Pont_Pro::init();

endif;
