<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class VP_Woo_Pont_Packeta {
	protected $api_url = 'https://www.zasilkovna.cz/api/rest/';
	protected $api_key = '';
	protected $api_password = '';
	protected $sender_id = '';

	public function __construct() {
		$this->api_key = VP_Woo_Pont_Helpers::get_option('packeta_api_key');
		$this->api_password = VP_Woo_Pont_Helpers::get_option('packeta_api_password');
		$this->sender_id = VP_Woo_Pont_Helpers::get_option('packeta_sender');
	}

	public function create_label($data) {

		//Create a new XML object
		$packet = new SimpleXMLElement('<createPacket></createPacket>');

		//Set password
		$packet->addChild('apiPassword', $this->api_password);

		//Set packat attributes
		$attributes = $packet->addChild('packetAttributes');
		$attributes->addChild('number', $data['order_id']);
		$attributes->addChild('name', $data['customer']['first_name']);
		$attributes->addChild('surname', $data['customer']['last_name']);
		if($data['customer']['company']) {
			$attributes->addChild('company', $data['customer']['company']);
		}
		$attributes->addChild('email', $data['customer']['email']);
		$attributes->addChild('phone', $data['customer']['phone']);
		$attributes->addChild('addressId', $data['point_id']);
		$attributes->addChild('value', $data['package']['total']);
		$attributes->addChild('eshop_id', $this->sender_id);

		//Set default weight if not exists
		if($data['package']['weight'] == 0) {
			$attributes->addChild('weight', 1);
		} else {
			$attributes->addChild('weight', $data['package']['weight']);
		}

		//Check for COD
		if($data['package']['cod']) {
			$attributes->addChild('currency', $data['package']['currency']);
			$attributes->addChild('cod', $data['package']['total_rounded']);
		}

		//Convert to xml string
		$packet = apply_filters('vp_woo_pont_packeta_label', $packet, $data);
		$xml = $packet->asXML();

		//Logging
		VP_Woo_Pont()->log_debug_messages($packet, 'packeta-create-label');

		//Create APi request
		$request = wp_remote_post( $this->api_url, array(
			'body' => $xml
		));

		//Check for errors
		if(is_wp_error($request)) {
			return $request;
		}

		//Parse response
		$response = wp_remote_retrieve_body( $request );
		$response = json_encode(simplexml_load_string($response));
		$response = json_decode($response,true);

		//Check for errors
		if($response['status'] == 'fault') {
			VP_Woo_Pont()->log_error_messages($response, 'packeta-create-label');
			return new WP_Error( $response['fault'], $response['string'] );
		}

		//Else, it was successful
		$parcel_number = $response['result']['id'];

		//Next, generate the PDF label
		$packet_label = new SimpleXMLElement('<packetLabelPdf></packetLabelPdf>');

		//Set password and packet id
		$packet_label->addChild('apiPassword', $this->api_password);
		$packet_label->addChild('packetId', $parcel_number);
		$packet_label->addChild('format', VP_Woo_Pont_Helpers::get_option('packeta_label_format', 'A7 on A4'));
		$packet_label->addChild('offset', '0');

		//Convert to xml string
		$xml = $packet_label->asXML();

		//Create APi request
		$request = wp_remote_post( $this->api_url, array(
			'body' => $xml
		));

		//Check for errors
		if(is_wp_error($request)) {
			return $request;
		}

		//Parse response
		$response = wp_remote_retrieve_body( $request );
		$response = json_encode(simplexml_load_string($response));
		$response = json_decode($response,true);

		//Check for errors
		if($response['status'] == 'fault') {
			VP_Woo_Pont()->log_error_messages($response, 'packeta-download-label');
			return new WP_Error( $response['fault'], $response['string'] );
		}

		//Now we have the PDF as base64, save it
		$attachment = $response['result'];
		$pdf = base64_decode($attachment);

		//Try to save PDF file
		$pdf_file = VP_Woo_Pont_Labels::get_pdf_file_path('packeta', $data['order_id']);
		VP_Woo_Pont_Labels::save_pdf_file($pdf, $pdf_file);

		//Create response
		$label = array();
		$label['id'] = $parcel_number;
		$label['number'] = $parcel_number;
		$label['pdf'] = $pdf_file['name'];

		//Return file name, package ID, tracking number which will be stored in order meta
		return $label;

	}

	public function void_label($data) {

		//Create a new XML object
		$packet = new SimpleXMLElement('<cancelPacket></cancelPacket>');

		//Set password
		$packet->addChild('apiPassword', $this->api_password);

		//Set package ID
		$packet->addChild('packetId', $data['parcel_id']);

		//Convert to xml string
		$packet = apply_filters('vp_woo_pont_packeta_void_label', $packet, $data);
		$xml = $packet->asXML();

		//Create APi request
		$request = wp_remote_post( $this->api_url, array(
			'body' => $xml
		));

		//Check for errors
		if(is_wp_error($request)) {
			return $request;
		}

		//Parse response
		$response = wp_remote_retrieve_body( $request );
		$response = json_encode(simplexml_load_string($response));
		$response = json_decode($response,true);

		//Check for errors
		if($response['status'] == 'fault') {
			VP_Woo_Pont()->log_error_messages($response, 'packeta-delete-label');
			return new WP_Error( $response['fault'], $response['string'] );
		}

		//Check for success
		$label = array();
		$label['success'] = true;

		return $label;
	}

	//Return tracking link
	public function get_tracking_link($parcel_number) {
		return 'https://tracking.packeta.com/hu/?id='.esc_attr($parcel_number);
	}

}
