<?php
defined( 'ABSPATH' ) || exit;

$pro_required = false;
$pro_icon = false;
if(!VP_Woo_Pont_Helpers::is_pro_enabled()) {
	$pro_required = true;
	$pro_icon = '<i class="vp_woo_pont_pro_label">PRO</i>';
}

$settings = array(
	'section_general' => array(
		'title' => __( 'General settings', 'vp-woo-pont' ),
		'type' => 'vp_woo_pont_settings_title',
		'description' => __( 'You need to take a look at these option at least to get started.', 'vp-woo-pont' ),
	),
	'title'      => array(
		'title'       => __( 'Method title', 'vp-woo-pont' ),
		'type'        => 'text',
		'description' => __( 'This controls the title which the user sees during checkout.', 'vp-woo-pont' ),
		'default'     => _x( 'Pickup point', 'frontend', 'vp-woo-pont' ),
		'desc_tip'    => true,
	),
	'tax_status' => array(
		'title'   => __( 'Tax status', 'vp-woo-pont' ),
		'type'    => 'select',
		'class'   => 'wc-enhanced-select',
		'default' => 'taxable',
		'options' => array(
			'taxable' => __( 'Taxable', 'vp-woo-pont' ),
			'none'    => _x( 'None', 'Tax status', 'vp-woo-pont' ),
		),
	),
	'enabled_providers' => array(
		'type' => 'vp_woo_pont_settings_checkboxes',
		'title' => __( 'Enabled providers', 'vp-woo-pont' ),
		'options' => $this->get_available_providers(),
		'default' => array('custom', 'foxpost', 'postapont')
	),
	'custom_title'      => array(
		'title'       => __( 'Custom provider name', 'vp-woo-pont' ),
		'type'        => 'text',
		'description' => __( 'If you setup your own pickup points, this will be the name of that category.', 'vp-woo-pont' ),
		'default'     => __( 'Store Pickup', 'vp-woo-pont' ),
		'desc_tip'    => true,
	),
	'cost'       => array(
		'title'             => __( 'Default cost', 'vp-woo-pont' ),
		'type'              => 'text',
		'placeholder'       => '',
		'description'       => __('Enter a default price for this shipping option. You can overwrite this later based on conditional logic.', 'vp-woo-pont'),
		'default'           => '0',
		'desc_tip'          => true,
		'sanitize_callback' => array( $this, 'sanitize_cost' ),
	),
	'detailed_cost' => array(
		'title' => __('Detailed cost', 'vp-woo-pont'),
		'type' => 'vp_woo_pont_settings_pricing_table'
	),
	'cost_logic' => array(
		'title'   => __( 'Multiple cost logic', 'vp-woo-pont' ),
		'type'    => 'select',
		'class'   => 'wc-enhanced-select',
		'default' => 'low',
		'description'       => __('If theres multiple matches for the shipping cost, use the lowest or the highest cost.', 'vp-woo-pont'),
		'options' => array(
			'low' => __( 'Lowest', 'vp-woo-pont' ),
			'high'    => _x( 'Highest', 'Tax status', 'vp-woo-pont' ),
		),
	),
	'points_manager' => array(
		'title' => _x('Pickup points', 'admin', 'vp-woo-pont'),
		'type' => 'vp_woo_pont_settings_points_table',
		'description' => __('You can setup your own pickup points too. Make sure you enter a unique ID(used internally only). These points will show up under the "Custom Provider" name.', 'vp-woo-pont')
	),
	'debug' => array(
		'title' => __( 'Developer mode', 'vp-woo-pont' ),
		'type' => 'checkbox',
		'desc_tip' => __( 'If turned on, the data sent to the provider during when createing a label will be logged in WooCommerce / Status / Logs. Can be used to debug issues.', 'vp-woo-pont' ),
	),
	'section_automation' => array(
		'title' => __( 'Label generator', 'vp-woo-pont' ).$pro_icon,
		'type' => 'vp_woo_pont_settings_title',
		'description' => __( 'Settings related to automations and labels. Every function in this section will only work if you have the PRO version activated.', 'vp-woo-pont' ),
	),
	'automations' => array(
		'type' => 'vp_woo_pont_settings_automations',
		'title' => __('Automations', 'vp-woo-pont')
	),
	'email_tracking_number' => array(
		'type' => 'multiselect',
		'title' => __( 'Tracking information in e-mails', 'vp-woo-pont' ),
		'class' => 'wc-enhanced-select',
		'default' => array('customer_completed_order'),
		'options' => array(),
		'description' => __('Selected the e-mails where you want to include the tracking number and tracking link.', 'vp-woo-pont').'<span id="vp_woo_pont_load_email_ids_nonce" data-nonce="'.wp_create_nonce("vp_woo_pont_load_email_ids").'"></span>'
	),
	'email_tracking_number_pos' => array(
		'type' => 'select',
		'class' => 'wc-enhanced-select',
		'title' => __( 'E-mail text position', 'vp-woo-pont' ),
		'desc_tip' => __( 'Where should the tracking numbers be included in the emails?', 'vp-woo-pont' ),
		'default' => 'beginning',
		'options' => array(
			'beginning' => __( 'At the beginning', 'vp-woo-pont' ),
			'end' => __( 'At the end', 'vp-woo-pont' ),
		),
	),
	'email_tracking_number_desc' => array(
		'title' => __( 'E-mail text', 'vp-woo-pont' ),
		'type' => 'textarea',
		'placeholder' => __('You can track the order by clicking on the tracking number: {tracking_number}', 'vp-woo-pont'),
		'desc_tip' => __('This text will be included in the selected e-mails. Use the {tracking_number} replacement code for the actual tracking link. This will be replaced with the package number as a link, redirecting to the providers tracking page.', 'vp-woo-pont')
	),
	'bulk_download_zip' => array(
		'title'    => __( 'Create a ZIP file during bulk download', 'vp-woo-pont' ),
		'type'     => 'checkbox',
		'disabled' => (!class_exists('ZipArchive')),
		'desc_tip' => __( 'If you want to download multiple shipping labels at once, this option will create a ZIP file with separate PDF files(the default option will merge all invoices into a single PDF).', 'vp-woo-pont' ),
		'description' => $this->get_bulk_zip_error()
	),

	'section_packeta' => array(
		'title' => __( 'Packeta settings', 'vp-woo-pont' ),
		'type' => 'vp_woo_pont_settings_title',
		'description' => __( 'Settings related to the Packeta provider.', 'vp-woo-pont' ),
	),
	'packeta_api_key' => array(
		'title' => __('Packeta API key', 'vp-woo-pont'),
		'type' => 'text',
		'description' => __('Please enter your Packeta API key if you plan to use it. This is required to get an up to date list of pickup points. Sign in into your Packeta account, click on your name top right and you can find the API key there.', 'vp-woo-pont')
	),
	'packeta_api_password' => array(
		'title' => __('Packeta API password', 'vp-woo-pont'),
		'type' => 'text',
		'description' => __('Please enter your Packeta API password. This is required to generate labels for packages. Sign in into your Packeta account, click on your name top right and you can find the API key password there.', 'vp-woo-pont')
	),
	'packeta_sender' => array(
		'title' => __('Sender ID', 'vp-woo-pont'),
		'type' => 'text',
		'description' => __('Please enter your SenderID. Sign in into your Packeta account and click User Informations / Senders.', 'vp-woo-pont')
	),
	'packeta_label_format' => array(
		'type' => 'select',
		'class' => 'wc-enhanced-select',
		'title' => __( 'Label format', 'vp-woo-pont' ),
		'default' => 'XS',
		'options' => array(
			'A6 on A6' => __( 'A6 on A6', 'vp-woo-pont' ),
			'A7 on A7' => __( 'A7 on A7', 'vp-woo-pont' ),
			'A6 on A4' => __( 'A6 on A4', 'vp-woo-pont' ),
			'A7 on A4' => __( 'A7 on A4', 'vp-woo-pont' ),
			'105x35mm on A4' => __( '105x35mm on A4', 'vp-woo-pont' ),
			'A8 on A8' => __( 'A8', 'vp-woo-pont' ),
		),
	),
	'section_foxpost' => array(
		'title' => __( 'Foxpost settings', 'vp-woo-pont' ),
		'type' => 'vp_woo_pont_settings_title',
		'description' => __( 'Settings related to the Foxpost provider.', 'vp-woo-pont' ),
	),
	'foxpost_username' => array(
		'title' => __('Foxpost e-mail', 'vp-woo-pont'),
		'type' => 'text',
		'description' => __("Enter your Foxpost account's e-mail address.", 'vp-woo-pont')
	),
	'foxpost_webhook_url' => array(
		'title' => __( 'Foxpost Webhook URL', 'vp-woo-pont' ),
		'type' => 'vp_woo_pont_settings_webhook',
		'default' => $this->get_webhook_url('foxpost'),
		'custom_attributes' => array(
			'readonly' => 'readonly'
		),
		'description' => __( 'Your webshop can get notified when a label has been created or when the tracking status changed. Sign in into your Foxpost account and enter this URL on the Settings / Webhook URL field.', 'vp-woo-pont' )
	),
	'foxpost_api_key' => array(
		'title' => __('Foxpost API key', 'vp-woo-pont'),
		'type' => 'text',
		'description' => __('Please enter your Foxpost API key. This is required for to generate a label. Sign in into your Foxpost account, click Settings and enter an API key there. Use the same value in this field.', 'vp-woo-pont')
	),
	'foxpost_parcel_size' => array(
		'type' => 'select',
		'class' => 'wc-enhanced-select',
		'title' => __( 'Parcel size', 'vp-woo-pont' ),
		'default' => 'XS',
		'options' => array(
			'XS' => __( 'XS', 'vp-woo-pont' ),
			'S' => __( 'S', 'vp-woo-pont' ),
			'M' => __( 'M', 'vp-woo-pont' ),
			'L' => __( 'L', 'vp-woo-pont' ),
			'XL' => __( 'XL', 'vp-woo-pont' ),
		),
	),
	'foxpost_sticker_size' => array(
		'type' => 'select',
		'class' => 'wc-enhanced-select',
		'title' => __( 'Sticker size', 'vp-woo-pont' ),
		'default' => 'A7',
		'options' => array(
			'a7' => __( 'A7', 'vp-woo-pont' ),
			'a5' => __( 'A5', 'vp-woo-pont' ),
		),
	),
	'section_gls' => array(
		'title' => __( 'GLS settings', 'vp-woo-pont' ),
		'type' => 'vp_woo_pont_settings_title',
		'description' => __( 'Settings related to the GLS provider.', 'vp-woo-pont' ),
	),
	'gls_username' => array(
		'title' => __('GLS Username', 'vp-woo-pont'),
		'type' => 'text',
		'description' => __("Enter your GLS account's e-mail address.", 'vp-woo-pont')
	),
	'gls_password' => array(
		'title' => __('GLS Password', 'vp-woo-pont'),
		'type' => 'text',
		'description' => __("Enter your GLS account's password.", 'vp-woo-pont')
	),
	'gls_client_id' => array(
		'title' => __('GLS Customer Number', 'vp-woo-pont'),
		'type' => 'text',
		'description' => __("Enter your GLS customer number.", 'vp-woo-pont')
	),
	'gls_dev_mode' => array(
		'title'    => __( 'Enable DEV mode', 'vp-woo-pont' ),
		'type'     => 'checkbox'
	),
	'gls_sender_name' => array(
		'title' => __('Sender name', 'vp-woo-pont'),
		'type' => 'text',
	),
	'gls_sender_street' => array(
		'title' => __('Sender address(street)', 'vp-woo-pont'),
		'type' => 'text',
	),
	'gls_sender_address' => array(
		'title' => __('Sender address(just the number)', 'vp-woo-pont'),
		'type' => 'text',
	),
	'gls_sender_address_2' => array(
		'title' => __('Sender address info(building, stairway, etc...)', 'vp-woo-pont'),
		'type' => 'text',
	),
	'gls_sender_city' => array(
		'title' => __('Sender city', 'vp-woo-pont'),
		'type' => 'text',
	),
	'gls_sender_postcode' => array(
		'title' => __('Sender postcode', 'vp-woo-pont'),
		'type' => 'text',
	),
	'gls_sender_phone' => array(
		'title' => __('Sender phone number', 'vp-woo-pont'),
		'type' => 'text',
	),
	'gls_sender_email' => array(
		'title' => __('Sender email address', 'vp-woo-pont'),
		'type' => 'text',
		'default' => get_bloginfo('admin_email')
	),
	'gls_package_contents' => array(
		'title' => __('Package contents text', 'vp-woo-pont'),
		'type' => 'textarea',
		'description' => __('You can use the following shortcodes: {order_number}, {order_items}, {customer_note}', 'vp-woo-pont')
	),
	'gls_sticker_size' => array(
		'type' => 'select',
		'class' => 'wc-enhanced-select',
		'title' => __( 'Sticker size', 'vp-woo-pont' ),
		'default' => 'A4_2x2',
		'options' => array(
			'A4_2x2' => __( 'A4_2x2', 'vp-woo-pont' ),
			'A4_4x1' => __( 'A4_4x1', 'vp-woo-pont' ),
			'Connect' => __( 'Connect', 'vp-woo-pont' ),
			'Thermo' => __( 'Thermo', 'vp-woo-pont' ),
		),
	),
	'gls_extra_services' => array(
		'type' => 'multiselect',
		'title' => __( 'Enabled services', 'vp-woo-pont' ),
		'class' => 'wc-enhanced-select',
		'default' => array(),
		'options' => array(
			'FDS' => __('Flexible Delivery Service', 'vp-woo-pont'),
			'CS1' => __('Contact Service', 'vp-woo-pont'),
			'SM2' => __('SMS service', 'vp-woo-pont'),
		)
	),
);

return $settings;
