<?php
/*
Plugin Name: Hungarian Pickup Points for WooCommerce
Plugin URI: http://visztpeter.me
Description: Csomagpontos szállítási módok WooCommerce webáruházhoz
Author: Viszt Péter
Version: 1.4.2
WC requires at least: 5.0
WC tested up to: 5.5.2
*/

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class VP_Woo_Pont {
	public static $plugin_prefix;
	public $plugin_url;
	public static $plugin_path;
	public static $plugin_basename;
	public static $version;
	public $template_base = null;
	public $settings = null;
	protected static $_instance = null;
	public static $sidebar_loaded = false;
	public $labels = null;
	public static $appsero;
	public static $license;

	//Load providers
	public $providers = array();

	//Ensures only one instance of class is loaded or can be loaded
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	//Just for a little extra security
	public function __clone() {
		_doing_it_wrong( __FUNCTION__, __( 'Cloning this object is forbidden.', 'vp-woo-pont' ) );
	}

	//Just for a little extra security
	public function __wakeup() {
		_doing_it_wrong( __FUNCTION__, __( 'Unserializing instances of this class is forbidden.', 'vp-woo-pont' ) );
	}

	//Construct
	public function __construct() {

		//Register activation hook
		register_activation_hook( __FILE__, array( $this, 'on_activate') );

		//Runs when plugin is successfulyl activated
		add_action( 'activated_plugin', array( $this, 'on_activate_success') );

		//Default variables
		self::$plugin_prefix = 'vp_woo_pont';
		self::$plugin_basename = plugin_basename(__FILE__);
		self::$plugin_path = trailingslashit(dirname(__FILE__));
		self::$version = '1.4.2';
		$this->plugin_url = plugin_dir_url(self::$plugin_basename);

		//Setup appsero for activation and updates
		if ( ! class_exists( 'Appsero\Client' ) ) {
			require_once __DIR__ . '/appsero/src/Client.php';
		}
		self::$appsero = new Appsero\Client( 'fdc2f165-b64b-4287-8789-e86fa9722611', 'VP Woo Pont', __FILE__ );

		// Active automatic updater
		self::$appsero->updater();

		//Set license check
		self::$license = self::$appsero->license();

		//Helper functions & classes
		require_once( plugin_dir_path( __FILE__ ) . 'includes/class-helpers.php' );
		require_once( plugin_dir_path( __FILE__ ) . 'includes/class-conditions.php' );
		require_once( plugin_dir_path( __FILE__ ) . 'includes/class-pro.php' );
		require_once( plugin_dir_path( __FILE__ ) . 'includes/class-import.php' );
		require_once( plugin_dir_path( __FILE__ ) . 'includes/class-customizer.php' );
		require_once( plugin_dir_path( __FILE__ ) . 'includes/class-walkthrough.php' );
		require_once( plugin_dir_path( __FILE__ ) . 'includes/class-automations.php' );

		//Init label generators
		require_once( plugin_dir_path( __FILE__ ) . 'includes/class-labels.php' );
		$this->labels = new VP_Woo_Pont_Labels();

		//Check if pro enabled
		$is_pro = VP_Woo_Pont_Helpers::is_pro_enabled();

		//Include compatibility modules if pro enabled
		if($is_pro) {
			require_once( plugin_dir_path( __FILE__ ) . 'includes/compatibility/class-compatibility.php' );
			VP_Woo_Pont_Compatibility::instance();
		}

		//Load provider classes
		$this->load_provider_classes();

		//Plugin loaded
		add_action( 'plugins_loaded', array( $this, 'init' ) );

		//Admin assets
		add_action( 'admin_init', array( $this, 'admin_js' ) );

		//Frontend assets
		add_action( 'wp_enqueue_scripts', array( $this, 'frontend_js' ));

		// Load admin notices
		if(is_admin()) {
			require_once( plugin_dir_path( __FILE__ ) . 'includes/class-admin-notices.php' );
		}

		//Plugin links
		add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( $this, 'plugin_action_links' ) );

		//Create new shipping method
		add_action( 'woocommerce_shipping_init', array( $this, 'shipping_init' ) );
		add_filter( 'woocommerce_shipping_methods', array( $this, 'add_method' ) );

		//Customize admin screen design and layout
		add_action( 'woocommerce_sections_shipping', array($this, 'wrap_start'), 9 );
		add_action( 'woocommerce_settings_shipping', array($this, 'wrap_end'), 20 );

		//Add custom UI to checkout page
		add_action( 'woocommerce_review_order_after_shipping', array($this, 'checkout_ui'));
		add_action( 'wp_footer', array($this, 'checkout_map_ui'));

		//Add custom UI to cart page
		add_action( 'woocommerce_cart_totals_before_order_total', array($this, 'cart_ui'));

		//Function when a pont is selected
		add_action( 'wp_ajax_vp_woo_pont_select', array( $this, 'select_pont_with_ajax' ) );
		add_action( 'wp_ajax_nopriv_vp_woo_pont_select', array( $this, 'select_pont_with_ajax' ) );

		//Change shipping option label on the checkout page
		add_filter( 'woocommerce_cart_shipping_method_full_label', array( $this, 'change_shipping_method_label' ), 10, 2);

		//Remove shipping address option if a point is selected
		add_filter( 'woocommerce_cart_needs_shipping_address', array( $this, 'hide_shipping_address' ));

		//Validate the vat number on checkout
		add_action( 'woocommerce_after_checkout_validation', array( $this, 'validate_checkout' ), 10, 2);

		//Saves the value to order meta
		add_action( 'woocommerce_checkout_update_order_meta', array( $this, 'save_checkout' ) );

		//Display the selected provider in the formatted shipping address
		add_filter( 'woocommerce_order_get_formatted_shipping_address', array( $this, 'format_shipping_address'), 10, 3);

		//Ajax function to remove selected point from order
		add_action( 'wp_ajax_vp_woo_pont_remove_point', array( $this, 'remove_pont_from_order' ) );
		add_action( 'wp_ajax_vp_woo_pont_replace_point', array( $this, 'replace_pont_in_order' ) );

		//Filter orders bassed on provider type
		add_action( 'restrict_manage_posts', array( $this, 'display_provider_filter' ) );
		add_action( 'pre_get_posts', array( $this, 'process_provider_filter' ) );

		//Import and export functions
		add_action( 'wp_ajax_vp_woo_pont_export_settings', array( $this, 'export_settings' ) );
		add_action( 'wp_ajax_vp_woo_pont_import_settings', array( $this, 'import_settings' ) );

		//Runs when a shipping method is selected
		add_action( 'woocommerce_checkout_update_order_review', array( $this, 'shipping_method_selected' ));

		//Get email ids
		add_action( 'wp_ajax_vp_woo_pont_get_email_ids', array( $this, 'get_email_ids_with_ajax' ) );

	}

	//When plugin is activated
	public function on_activate() {
		$upload_dir = wp_upload_dir();

		$files = array(
			array(
				'base' => $upload_dir['basedir'] . '/vp-woo-pont-db',
				'file' => 'index.html',
				'content' => ''
			),
			array(
				'base' => $upload_dir['basedir'] . '/vp-woo-pont-labels',
				'file' => 'index.html',
				'content' => ''
			)
		);

		foreach ( $files as $file ) {
			if ( wp_mkdir_p( $file['base'] ) && ! file_exists( trailingslashit( $file['base'] ) . $file['file'] ) ) {
				if ( $file_handle = @fopen( trailingslashit( $file['base'] ) . $file['file'], 'w' ) ) {
					fwrite( $file_handle, $file['content'] );
					fclose( $file_handle );
				}
			}
		}

		//Download json files on activation
		VP_Woo_Pont_Import_Database::schedule_actions();

	}

	public function on_activate_success($plugin) {
		//Redirect to setup wizard if not run already
		if(!get_option('_vp_woo_pont_walkthrough_finished') && $plugin == 'vp-woo-pont/index.php') {
			exit( wp_redirect( admin_url( 'options.php?page=vp-woo-pont-walkthrough' ) ) );
		}
	}

	//Load provider classes
	public function load_provider_classes() {
		include_once( plugin_dir_path( __FILE__ ) . 'includes/providers/class-foxpost.php' );
		include_once( plugin_dir_path( __FILE__ ) . 'includes/providers/class-packeta.php' );
		include_once( plugin_dir_path( __FILE__ ) . 'includes/providers/class-gls.php' );
		$this->providers['foxpost'] = new VP_Woo_Pont_Foxpost();
		$this->providers['packeta'] = new VP_Woo_Pont_Packeta();
		$this->providers['gls'] = new VP_Woo_Pont_GLS();
	}

	//Load locale
	public function init() {
		load_plugin_textdomain( 'vp-woo-pont', false, basename( dirname( __FILE__ ) ) . '/languages/' );
	}

	//Add Admin CSS & JS
	public function admin_js() {
		wp_enqueue_script( 'vp_woo_pont_print_js', plugins_url( '/assets/js/print.min.js',__FILE__ ), array('jquery'), VP_Woo_Pont::$version, TRUE );
		wp_enqueue_script( 'vp_woo_pont_admin_js', plugins_url( '/assets/js/admin.min.js',__FILE__ ), array('jquery', 'wc-backbone-modal'), VP_Woo_Pont::$version, TRUE );
		wp_enqueue_style( 'vp_woo_pont_admin_css', plugins_url( '/assets/css/admin.css',__FILE__ ), array(), VP_Woo_Pont::$version );

		$vp_woo_pont_local = array(
			'loading' => plugins_url( '/assets/images/ajax-loader.gif',__FILE__ ),
			'settings_link' => esc_url(admin_url( 'admin.php?page=wc-settings&tab=vp_woo_pont' )),
			'providers' => VP_Woo_Pont_Helpers::get_supported_providers()
		);

		//Load json file urls
		$json_files = VP_Woo_Pont_Helpers::get_json_files();
		$vp_woo_pont_local['files'] = $json_files;

		wp_localize_script( 'vp_woo_pont_admin_js', 'vp_woo_pont_params', $vp_woo_pont_local );
	}

	//Frontend CSS & JS
	public function frontend_js() {

		//Check if we need to show it on the cart page too
		$show_in_cart = false;
		if(is_cart() && get_option('vp_woo_pont_show_on_cart', 'yes') == 'yes') {
			$show_in_cart = true;
		}

		//Only on the checkout page
		if(is_checkout() || $show_in_cart) {
			wp_enqueue_style( 'vp_woo_pont_frontend_css', plugins_url( '/assets/css/frontend.css',__FILE__ ), array(), VP_Woo_Pont::$version );
			wp_enqueue_script( 'vp_woo_pont_frontend_js', plugins_url( '/assets/js/frontend.min.js',__FILE__ ), array('jquery'), VP_Woo_Pont::$version );
			$vp_woo_pont_local = array(
				'files' => VP_Woo_Pont_Helpers::get_json_files(),
				'custom_points' => VP_Woo_Pont_Helpers::get_option('vp_woo_pont_points'),
				'enabled_providers' => VP_Woo_Pont_Helpers::get_option('vp_woo_pont_enabled_providers'),
				'prices_including_tax' => WC()->cart->display_prices_including_tax(),
				'refresh_payment_methods' => VP_Woo_Pont_Helpers::pricing_has_payment_method_condition()
			);
			wp_localize_script( 'vp_woo_pont_frontend_js', 'vp_woo_pont_frontend_params', $vp_woo_pont_local );
		}

	}

	//Plugin links
	public function plugin_action_links( $links ) {
		$action_links = array(
			'settings' => '<a href="' . esc_url(admin_url( 'admin.php?page=wc-settings&tab=shipping&section=vp_pont' )) . '" aria-label="' . esc_attr__( 'VP Woo Pont Settings', 'vp-woo-pont' ) . '">' . esc_html__( 'Settings', 'vp-woo-pont' ) . '</a>',
			'documentation' => '<a href="https://visztpeter.me/dokumentacio/" target="_blank" aria-label="' . esc_attr__( 'VP Woo Pont Documentation', 'vp-woo-pont' ) . '">' . esc_html__( 'Documentation', 'vp-woo-pont' ) . '</a>'
		);

		if (!VP_Woo_Pont_Helpers::is_pro_enabled() ) {
			$action_links['get-pro'] = '<a target="_blank" rel="noopener noreferrer" style="color:#46b450;" href="https://visztpeter.me/vp-woo-pont" aria-label="' . esc_attr__( 'VP Woo Pont Pro version', 'vp-woo-pont' ) . '">' . esc_html__( 'Pro version', 'vp-woo-pont' ) . '</a>';
		}

		return array_merge( $action_links, $links );
	}

	//Load gateway class
	public function shipping_init() {
		include_once __DIR__ . '/includes/class-wc-shipping-pont.php';
	}

	//Add shipping method
	public function add_method( $methods ) {
		$methods['vp_pont'] = 'WC_Shipping_Pont';
		return $methods;
	}

	//Wrap page content into divs
	public function wrap_start() {
		if($this->is_settings_page()) {
			echo '<div class="vp-woo-pont-settings-wrapper">';
			echo '<div class="vp-woo-pont-settings-content">';
		}
	}

	//End wrap and display sidebar
	public function wrap_end() {
		if($this->is_settings_page() && !self::$sidebar_loaded) {
			self::$sidebar_loaded = true;
			echo '</div>';
			include( dirname( __FILE__ ) . '/includes/views/html-admin-sidebar.php' );
			echo '</div>';
		}
	}

	//Check if we are on the settings page
	public function is_settings_page() {
		global $current_section;
		$is_settings_page = false;
		if( isset( $_GET['page'], $_GET['tab'], $_GET['section'] ) && 'wc-settings' === $_GET['page'] && 'shipping' === $_GET['tab'] && 'vp_pont' === $_GET['section'] ) {
			$is_settings_page = true;
		}
		return $is_settings_page;
	}

	//Render custom html for the checkout page
	public function checkout_ui() {

		//Get selected shipping methd
		$chosen_methods = WC()->session->chosen_shipping_methods;

		//If vp_pont is chosen
		$is_vp_pont_selected = false;
		foreach ($chosen_methods as $chosen_method) {
			if(strpos($chosen_method, 'vp_pont') !== false) {
				$is_vp_pont_selected = true;
			}
		}

		//Get selected pont
		$selected_pont = WC()->session->get( 'selected_vp_pont' );

		//Get shipping cost
		$shipping_costs = VP_Woo_Pont_Helpers::calculate_shipping_costs();
		$shipping_cost = VP_Woo_Pont_Helpers::get_shipping_cost();

		//Use wc_get_template, so it can be overwritten from a theme if needed
		wc_get_template('checkout/pont.php', array(
			'is_vp_pont_selected' => $is_vp_pont_selected,
			'selected_vp_pont' => $selected_pont,
			'shipping_costs' => $shipping_costs,
			'shipping_cost' => $shipping_cost
		), false, VP_Woo_Pont::$plugin_path . '/templates/');
	}

	//Render custom html for the map modal window on the checkout page and cart page
	public function checkout_map_ui() {
		if(is_cart() || is_checkout()) {
			wc_get_template('checkout/pont-map.php', array(), false, VP_Woo_Pont::$plugin_path . '/templates/');
		}
	}

	public function find_point_info($provider, $point_id) {

		//If its a postapont, just use that as the provider for the json files
		$provider_json = $provider;
		if(strpos($provider, 'postapont') !== false) {
			$provider_json = 'postapont';
		}

		//Get submitted data
		$points = array();
		$download_folders = VP_Woo_Pont_Helpers::get_download_folder();
		$point = false;

		//Get the JSON file based on the provider type
		if($provider_json != 'custom') {
			$filename = get_option('_vp_woo_pont_file_'.$provider_json);
			$filepath = $download_folders['dir'].$filename;
			$json_file = file_get_contents($filepath);

			//Check if file exists
			if($json_file === false) {
				return false;
			}

			//Check if its a valid json file
			$json = json_decode($json_file, true);
			if ($json === null) {
				return false;
			}

			//Set points to find by id
			$points = $json;

		} else {

			//For custom, just load the stored points
			$points = VP_Woo_Pont_Helpers::get_option('vp_woo_pont_points');

		}

		//Find the point with the ID
		foreach ($points as $single_point) {
			if($single_point['id'] == $point_id) {
				$point = $single_point;
				break;
			}
		}

		//Check if we have a point found
		if($point) {

			//Set provider ID just in case
			$point['provider'] = $provider;
			$point['provider_json'] = $provider_json;

		}

		//Return point or false
		return $point;

	}

	//Runs when a point is selected from the map
	public function select_pont_with_ajax() {

		//Use the already existing woocommerce nonce, so we don't need to create one just for this
		if($_POST['page'] == 'checkout') {
			check_ajax_referer( 'update-order-review', 'security' );
		} else {
			check_ajax_referer( 'update-shipping-method', 'security' );
		}

		//Get submitted data
		$provider = sanitize_text_field($_POST['provider']);
		$provider_json = $provider;
		$id = sanitize_text_field($_POST['id']);
		$points = array();
		$download_folders = VP_Woo_Pont_Helpers::get_download_folder();
		$point = false;

		//If its a postapont, just use that as the provider for the json files
		if(strpos($provider, 'postapont') !== false) {
			$provider_json = 'postapont';
		}

		//Get point data
		$point = $this->find_point_info($provider, $id);

		//Check if we have a point found
		if($point) {

			//Reset shipping cost cache
			$packages = WC()->cart->get_shipping_packages();
			foreach ($packages as $key => $value) {
					$shipping_session = "shipping_for_package_$key";
					unset(WC()->session->$shipping_session);
			}

			//Store it in the checkout session. Use session, because it will remember the selected point if the checkotu page is reloaded
			WC()->session->set('selected_vp_pont', $point);

			//And return the point, maybe processing is done in JS
			wp_send_json_success(array('point' => $point));

		} else {

			//Return error if it doesn't exists
			wp_send_json_error();

		}

	}

	//Change label
	public function change_shipping_method_label($label, $method) {

		//For our own method, include a custom label and icons
		if($method->method_id == 'vp_pont') {

			//Get shippign cost
			$shipping_cost = VP_Woo_Pont_Helpers::calculate_shipping_costs();

			//Create an array of providers, for icon classes
			$provider_icons = array();

			//Find the smallest cost
			$minimum_cost = false;
			$minimum_cost_count = array();
			$has_free_shipping = false;
			$min_cost_formatted = '';
			$min_cost_label = '';
			foreach ($shipping_cost as $provider => $array) {
				if($array['net'] == 0) {
					$has_free_shipping = true;
				} else {
					$minimum_cost_count[] = $array['net'];
					if (!$minimum_cost) {
						$minimum_cost = $array;
					} elseif ($array['net'] < $minimum_cost['net']) {
						$minimum_cost = $array;
					}
				}
				$provider_icons[] = '<i class="vp-woo-pont-provider-icon-'.$provider.'"></i>';
			}

			//Check how many different prices we have
			$minimum_cost_count = array_unique($minimum_cost_count);
			$minimum_cost_count = count($minimum_cost_count);

			//Minimum cost label
			if($minimum_cost) {
				if ( WC()->cart->display_prices_including_tax() ) {
					$min_cost_formatted = $minimum_cost['formatted_gross'];
				} else {
					$min_cost_formatted = $minimum_cost['formatted_net'];
				}
			}

			//Minimum cost label, only free shipping
			if($has_free_shipping && $minimum_cost_count == 0) {
				$min_cost_label = esc_html_x( 'free', 'shipping cost summary on cart & checkout', 'vp-woo-pont' );
			}

			//Minimum cost label, only 1 paid shipping
			if(!$has_free_shipping && $minimum_cost_count == 1) {
				$min_cost_label = sprintf( esc_html_x( '%s', 'shipping cost summary on cart & checkout(one shipping cost only)', 'vp-woo-pont' ), '<em class="vp-woo-pont-shipping-method-label-cost">' . $min_cost_formatted . '</em>' );
			}

			//Minimum cost label, multiple paid shipping
			if(!$has_free_shipping && $minimum_cost_count > 1) {
				$min_cost_label = sprintf( esc_html_x( 'from %s', 'shipping cost summary on cart & checkout(multiple shipping costs)', 'vp-woo-pont' ) . ' ', '<em class="vp-woo-pont-shipping-method-label-cost">' . $min_cost_formatted . '</em>' );
			}

			//Minimum cost label, free shipping + paid shipping
			if($has_free_shipping && $minimum_cost_count == 1) {
				$min_cost_label = sprintf( esc_html_x( 'free or %s', 'shipping cost summary on cart & checkout(free & 1 shipping cost)', 'vp-woo-pont' ) . ' ', '<em class="vp-woo-pont-shipping-method-label-cost">' . $min_cost_formatted . '</em>' );
			}

			//Minimum cost label, free shipping + paid shipping
			if($has_free_shipping && $minimum_cost_count > 1) {
				$min_cost_label = sprintf( esc_html_x( 'free & from %s', 'shipping cost summary on cart & checkout(free & 1+ shipping cost)', 'vp-woo-pont' ) . ' ', '<em class="vp-woo-pont-shipping-method-label-cost">' . $min_cost_formatted . '</em>' );
			}

			//Create new labels with price and optional icons
			$label = '<span class="vp-woo-pont-shipping-method-label">'.$method->get_label().': <span class="vp-woo-pont-shipping-method-label-price">'.$min_cost_label.'</span></span>';

			//If we need to display small icons below the label(based on customizer option)
			if(get_option('vp_woo_pont_small_icons', 'yes') == 'yes' && (is_checkout() || is_cart())) {
				$label .= '<span class="vp-woo-pont-shipping-method-icons">'.implode(' ', $provider_icons).'</span>';
			}

			//For developers to modify
			$label = apply_filters('vp_woo_pont_shipping_method_label', $label, $shipping_cost);

		}

		return $label;
	}

	public function hide_shipping_address($needs_shipping_address) {

		//Get selected shipping methd
		$chosen_methods = WC()->session->chosen_shipping_methods;

		//If vp_pont is chosen
		foreach ($chosen_methods as $chosen_method) {
			if(strpos($chosen_method, 'vp_pont') !== false) {
				$needs_shipping_address = false;
			}
		}

		return $needs_shipping_address;
	}

	public function validate_checkout($fields, $errors) {

		//If its the vp_pont shippign method
		$chosen_methods = WC()->session->get( 'chosen_shipping_methods' );
		$selected_pont = WC()->session->get( 'selected_vp_pont' );
	  $chosen_method = $chosen_methods[0];

		//Check if a a vp_pont is selected
		if(strpos($chosen_method, 'vp_pont') !== false && !$selected_pont) {
			$errors->add( 'validation', apply_filters('vp_woo_pont_required_pont_message', esc_html__( 'Please select a pick-up point or choose a different shipping method.', 'vp-woo-pont'), $fields) );
		}

	}

	public function save_checkout( $order_id ) {

		//If we have a pont selected, that means we need to save it
		$chosen_methods = WC()->session->get( 'chosen_shipping_methods' );
		$selected_pont = WC()->session->get( 'selected_vp_pont' );
		$chosen_method = $chosen_methods[0];

		//If shipping is vp_pont and a pont was selected, save its data as the shipping address(and some custom meta too)
		if(strpos($chosen_method, 'vp_pont') !== false && $selected_pont) {
			$order = wc_get_order( $order_id );

			//Save custom meta and replace shipping address
			$this->update_order_with_selected_point($order, $selected_pont);

			// Get the customer id
			$customer_id = $order->get_customer_id();

			//Save user meta if the customer was signed in
			if( ! empty($customer_id) && $customer_id != 0) {
				update_user_meta( $customer_id, '_vp_woo_pont_point_id', $selected_pont['provider'].'|'.$selected_pont['id'] );
			}

		}
	}

	public function update_order_with_selected_point($order, $point) {
		$address = array(
			'first_name' => '',
			'last_name'  => '',
			'company'    => $point['name'],
			'email'      => '',
			'phone'      => '',
			'address_1'  => $point['addr'],
			'address_2'  => '',
			'city'       => $point['city'],
			'state'      => '',
			'postcode'   => $point['zip'],
			'country'    => 'HU'
		);

		//Set shipping address to the same as the point's address
		$order->set_address( $address, 'shipping' );

		//Store provider and point ID as custom meta
		$order->update_meta_data( '_vp_woo_pont_provider', $point['provider'] );
		$order->update_meta_data( '_vp_woo_pont_point_id', $point['id'] );
		$order->update_meta_data( '_vp_woo_pont_point_name', $point['name'] );
		$order->update_meta_data( '_vp_woo_pont_point_coordinates', $point['lat'].';'.$point['lon'] );

		//And save the order
		$order->save();
	}

	public function format_shipping_address($address, $raw_address, $order) {
		$prepend = '';
		if($provider_id = $order->get_meta('_vp_woo_pont_provider')) {
			$providers = VP_Woo_Pont_Helpers::get_supported_providers();
			$address = $providers[$provider_id].'<br>'.$address;
		}
		return $address;
	}

	public function remove_pont_from_order() {

		//Security check
		check_ajax_referer( 'vp_woo_pont_manage', 'nonce' );

		//Get order id
		$order_id = intval($_POST['order']);
		$order = wc_get_order($order_id);

		//Remove from order
		$order->delete_meta_data( '_vp_woo_pont_provider' );
		$order->delete_meta_data( '_vp_woo_pont_point_id' );
		$order->delete_meta_data( '_vp_woo_pont_point_name' );
		$order->delete_meta_data( '_vp_woo_pont_point_coordinates' );
		$order->save();

		//Send success response
		wp_send_json_success();
	}

	public function replace_pont_in_order() {

		//Security check
		check_ajax_referer( 'vp_woo_pont_manage', 'nonce' );

		//Get order id
		$order_id = intval($_POST['order']);
		$order = wc_get_order($order_id);
		$provider = sanitize_text_field($_POST['provider']);
		$point_id = sanitize_text_field($_POST['point_id']);

		//Get point data
		$point = $this->find_point_info($provider, $point_id);

		//Save custom meta and replace shipping address
		$this->update_order_with_selected_point($order, $point);

		//Create response
		$response = array();
		$providers = VP_Woo_Pont_Helpers::get_supported_providers();
		$response['point_id'] = $point['id'];
		$response['point_name'] = $point['name'];
		$response['provider_label'] = $providers[$point['provider']];
		$response['provider'] = $point['provider'];

		//Send success response
		wp_send_json_success($response);
	}


	public function display_provider_filter(){
		global $pagenow, $post_type;
		if( 'shop_order' === $post_type && 'edit.php' === $pagenow ) {
			$supported_providers = VP_Woo_Pont_Helpers::get_supported_providers();
			$selected_filter = isset($_GET['vp_woo_pont_provider_filter']) ? sanitize_text_field($_GET['vp_woo_pont_provider_filter']) : '';
			?>
			<select name="vp_woo_pont_provider_filter">
				<option value=""><?php _e('Filter shipping providers', 'vp-woo-pont'); ?></option>
				<?php foreach ($supported_providers as $provider_id => $label): ?>
					<option value="<?php echo esc_attr($provider_id); ?>" <?php selected($provider_id, $selected_filter); ?>><?php echo esc_html($label); ?></option>
				<?php endforeach; ?>
			</select>
			<?php
		}
	}

	public function process_provider_filter( $query ) {
		global $pagenow;
		if ( $query->is_admin && $pagenow == 'edit.php' && isset( $_GET['vp_woo_pont_provider_filter'] ) && $_GET['vp_woo_pont_provider_filter'] != '' && $_GET['post_type'] == 'shop_order' ) {
			$meta_query = $query->get( 'meta_query' );
			if(!$meta_query) $meta_query = array();

			//Append to meta query
			$meta_query[] = array(
				'meta_key' => '_vp_woo_pont_provider',
				'value' => esc_attr( $_GET['vp_woo_pont_provider_filter'] ),
			);

			//Set meta-query
			$query->set( 'meta_query', $meta_query );
		}

		return $query;
	}

	//Render custom html for the checkout page
	public function cart_ui() {

		//Get selected shipping methd
		$chosen_methods = WC()->session->chosen_shipping_methods;

		//If vp_pont is chosen
		$is_vp_pont_selected = false;
		foreach ($chosen_methods as $chosen_method) {
			if(strpos($chosen_method, 'vp_pont') !== false) {
				$is_vp_pont_selected = true;
			}
		}

		//Get selected pont
		$selected_pont = WC()->session->get( 'selected_vp_pont' );

		//Get shipping cost
		$shipping_costs = VP_Woo_Pont_Helpers::calculate_shipping_costs();
		$shipping_cost = VP_Woo_Pont_Helpers::get_shipping_cost();

		//Use wc_get_template, so it can be overwritten from a theme if needed
		wc_get_template('checkout/pont.php', array(
			'is_vp_pont_selected' => $is_vp_pont_selected,
			'selected_vp_pont' => $selected_pont,
			'shipping_costs' => $shipping_costs,
			'shipping_cost' => $shipping_cost
		), false, VP_Woo_Pont::$plugin_path . '/templates/');
	}

	public function export_settings() {

		//Security check
		check_ajax_referer( 'vp_woo_pont_settings_export', 'nonce' );

		//Get saved data and convert it to a json string
		$type = sanitize_text_field($_POST['type']);
		$saved_values = array();
		if($type == 'points') {
			$saved_values = get_option('vp_woo_pont_points');
		} else {
			$saved_values = get_option('vp_woo_pont_pricing');
		}

		//Return response
		wp_send_json_success($saved_values);

	}

	public function import_settings() {

		//Security check
		check_ajax_referer( 'vp_woo_pont_settings_export', 'nonce' );

		//Check type
		$type = sanitize_text_field($_POST['type']);

		//Check for file
		if (isset($_FILES['file'])){
			$file = $_FILES['file']['tmp_name'];
			$data = file_get_contents($file);
			$array = json_decode($data, true );

			if($type == 'pricing') {
				//Save pricing options
				$prices = array();
				if ( $array ) {
					foreach ($array as $pricing_id => $pricing) {

						$cost = wc_clean($pricing['cost']);
						$prices[$pricing_id] = array(
							'cost' => $cost,
							'conditional' => false,
							'providers' => array()
						);

						//If theres conditions to setup
						$condition_enabled = isset($pricing['conditional']) ? true : false;
						$conditions = (isset($pricing['conditions']) && count($pricing['conditions']) > 0);
						if($condition_enabled && $conditions) {
							$prices[$pricing_id]['conditional'] = true;
							$prices[$pricing_id]['conditions'] = array();
							$prices[$pricing_id]['logic'] = wc_clean($pricing['logic']);

							foreach ($pricing['conditions'] as $condition) {
								$condition_details = array(
									'category' => wc_clean($condition['category']),
									'comparison' => wc_clean($condition['comparison']),
									'value' => wc_clean($condition['value'])
								);

								$prices[$pricing_id]['conditions'][] = $condition_details;
							}
						}

						//Save providers
						$providers = (isset($pricing['providers']) && count($pricing['providers']) > 0);
						if($providers) {
							foreach ($pricing['providers'] as $provider) {
								$prices[$pricing_id]['providers'][] = $provider;
							}
						}

					}
				}
				update_option( 'vp_woo_pont_pricing', $prices );
			}

			if($type == 'points') {
				//Save pricing options
				$points = array();
				if ( $array ) {
					foreach ($array as $point_id => $point) {

						$name = wc_clean($point['name']);
						$id = wc_clean($point['id']);
						$provider = wc_clean($point['provider']);
						$coordinates = wc_clean($point['coordinates']);
						$zip = wc_clean($point['zip']);
						$addr = wc_clean($point['addr']);
						$city = wc_clean($point['city']);
						$comment = wp_kses_post( trim( wp_unslash($point['comment']) ) );
						$hidden = isset($point['hidden']) ? true : false;

						//Convert coordinates
						$coordinates = explode(';', $coordinates);

						//Create new point
						$points[$point_id] = array(
							'name' => $name,
							'id' => $id,
							'provider' => $provider,
							'lat' => $coordinates[0],
							'lon' => $coordinates[1],
							'zip' => $zip,
							'addr' => $addr,
							'city' => $city,
							'comment' => $comment,
							'hidden' => $hidden,
						);

					}
				}
				update_option( 'vp_woo_pont_points', $points );
			}
		}

		//Return response
		wp_send_json_success();

	}

	function shipping_method_selected( $chosen_method ) {

		//Check if the user is signed in
		if(is_user_logged_in()) {
			$customer_id = get_current_user_id();
			$point_info = get_user_meta( $customer_id, '_vp_woo_pont_point_id', true );
			$selected_pont = WC()->session->get( 'selected_vp_pont' );

			//If a point is stored
			if($point_info && !$selected_pont) {
				$point_info = explode('|', $point_info);
				$provider = $point_info[0];
				$point_id = $point_info[1];

				//Get point data
				$point = $this->find_point_info($provider, $point_id);

				//Check if we have a point found
				if($point) {

					//Store it in the checkout session.
					WC()->session->set('selected_vp_pont', $point);

				}

			}

		}

	}

	//Get email ids
	public function get_email_ids_with_ajax() {
		check_ajax_referer( 'vp_woo_pont_load_email_ids', 'nonce' );
		$document_types = array('tracking_number');

		//Get registered emails
		$mailer = WC()->mailer();
		$email_templates = $mailer->get_emails();
		$emails = array();

		//Omit a few one thats not required at all
		$disabled = ['failed_order', 'customer_note', 'customer_reset_password', 'customer_new_account'];

		//Loop through each document type
		foreach ($document_types as $document_type) {

			//Get saved values
			$saved_values = VP_Woo_Pont_Helpers::get_option('email_'.$document_type);
			if(!$saved_values) $saved_values = array();

			//Create options
			$options = array();
			foreach ( $email_templates as $email ) {
				if(!in_array($email->id,$disabled)) {
					$options[] = array(
						'label' => $email->get_title(),
						'selected' => in_array($email->id, $saved_values),
						'id' => $email->id
					);
				}
			}

			//Return select + options
			$emails[] = array(
				'field' => $document_type,
				'options' => $options
			);
		}

		wp_send_json_success($emails);
	}

	//Log error message
	public function log_error_messages($error, $source) {
		$logger = wc_get_logger();
		$logger->error(
			$source.' - '.json_encode($error),
			array( 'source' => 'vp_woo_pont' )
		);
	}

	//Log debug messages
	public function log_debug_messages($data, $source, $force = false) {
		if(VP_Woo_Pont_Helpers::get_option('debug', 'no') == 'yes' || $force) {
			$logger = wc_get_logger();
			$logger->debug(
				$source.' - '.json_encode($data),
				array( 'source' => 'vp_woo_pont' )
			);
		}
	}

}

//WC Detection
if ( ! function_exists( 'is_woocommerce_active' ) ) {
	function is_woocommerce_active() {
		$active_plugins = (array) get_option( 'active_plugins', array() );

		if ( is_multisite() ) {
			$active_plugins = array_merge( $active_plugins, get_site_option( 'active_sitewide_plugins', array() ) );
		}

		return in_array( 'woocommerce/woocommerce.php', $active_plugins ) || array_key_exists( 'woocommerce/woocommerce.php', $active_plugins ) ;
	}
}

//Initialize, if woocommerce is active
if ( is_woocommerce_active() ) {
	function VP_Woo_Pont() {
		return VP_Woo_Pont::instance();
	}

	VP_Woo_Pont();
}
